part of circumstellar;

class LoggerContext {
  LoggerContext parent = null;

  LoggingImplementation _default = null;

  String name;

  void _internalInit() {
    this._default = new DefaultLoggingImplementation(this);
    this._loggingImplementations.add(_default);
  }

  LoggerContext.fromParent(this.parent, this.name) {
    this._internalInit();
  }
  LoggerContext.root(this.name) {
    this._internalInit();
  }

  void addImplementation(LoggingImplementation impl) {
    this._loggingImplementations.add(impl);
  }

  void removeImplementation(LoggingImplementation impl) {
    this._loggingImplementations.remove(impl);
  }

  void removeDefaultImplementation() {
    this.removeImplementation(this._default);
  }

  List<LoggingImplementation> _loggingImplementations = [];

  void log(LogLevel level, String comment) {
    _loggingImplementations.forEach((impl) {
      impl.log(level, comment);
    });
  }
}

abstract class LoggingImplementation {
  LoggerContext context;
  LoggingImplementation(this.context);
  void log(LogLevel level, String comment);
}

class DefaultLoggingImplementation implements LoggingImplementation {
  String logIndent = '    ';
  String buildLoggerTree() {
    if (context.parent == null) {
      return context.name;
    } else
      return this.buildLoggerTree() + ' -> ' + context.name;
  }

  LoggerContext context;

  DefaultLoggingImplementation(this.context);

  @override
  void log(LogLevel level, String comment) {
    print(
        '${level.toString().replaceAll('LogLevel.', '')}: [${this.buildLoggerTree()}] at ${new DateTime.now().toLocal().toString()}');
    print(this.logIndent + comment);
  }
}

enum LogLevel { Trace, Debug, Info, Warn, Error, Critical, Crash }
